#!/usr/bin/env bash
# <section>
#   <title>
#       <c>build-proposal-html</c>: Build the proposal as HTML
#   </title>
#
#   <introduction>
#       <p>
#           This script requires the <pretext/> repository and the
#           <c>xsltproc</c> command to run. It does not support
#           Windows (outside of WSL).
#       </p>
#   </introduction>
#
#   <subsection>
#       <title>Setup</title>
#
# <p>
#   Source a custom file with two path names, <c>PTX</c> and
#   <c>PRO</c>. See <xref ref="paths.sh.template">; you must copy
#   this file to <c>paths.sh</c> then edit it to set these paths
#   based on when they exist on your computer.
# </p>
#
# <p>
#   Get the path to the directory this script is located in; see
#   <url href="https://stackoverflow.com/a/59916">Stack
#   Overflow</url> (hereafter abbreviated SO).
# </p>
DIR="$(dirname "$0")"
# Note that <c>.</c> is bash shorthand for <c>source</c>. For
# alternatives, see <url
# href=http://stackoverflow.com/questions/192292">SO</url>.
. ${DIR}/paths.sh

# <p>
#   Compute a temporary output directory; per <xref
#   ref=".gitignore">, this directory isn't tracked by Git.
#   Therefore, you must manually copy desirable PDF versions into
#   the tracked <c>${PRO}/output/proposal</c>.
# </p>
#
# <p>
#   TODO: Why do we need to define <c>PRO</c> if we can compute
#   <c>DIR</c>?
#</p>
OUT="${PRO}/temp-output"

# Preparation: Use <url
# href="https://linux.die.net/man/1/install">install</url> to copy
# <pretext/> NSF proposal XSLT files into the <c>OUT</c> directory.
# TODO: Why is this needed?
install -d ${PTX}/user ${OUT}
# Overwrite the NSF XSLT files in the pretext repository with the
# custom version from this repository.
cp -a ${PRO}/xsl/nsf-proposal-html.xsl ${PTX}/user

# <p>
#   TODO: I don't understand these instructions.
#   copy images folder (include final dot!)
#   2022 not used yet, check managed directories
# </p>
#cp -a ${PRO}/src/images .
#
#   </subsection>
#
#   <subsection>
#       <title>Supporting functions</title>
#
# <p>
#   Print a banner to stderr, so it's clear where any errors for the
#   current build come from.
# </p>
#
# <p>
#   Parameters: any parameters (<c>$*</c>) are included in the
#   message.
# </p>
banner () {
    # Redirect echo's output to stderr. See <url
    # href="https://stackoverflow.com/a/23550347">SO</url>.
    >&2 echo "*******************************************************"
    >&2 echo "Building $*..."
    >&2 echo "*******************************************************"
}

# <p>Run <c>xsltproc</c> configured to build HTML output.</p>
#
# <p>Parameters:
#   <ul>
#       <li>$1: The <pretext/> source file to build.</li>
#       <li>
#           all other arguments: command-line options to pass to
#           <c>xsltproc</c>.
#       </li>
#   </ul>
# </p>
run_xsltproc () {
    # Per the <url href="http://xmlsoft.org/xslt/xsltproc.html"
    # >xsltproc docs</url>, note that all command-line parameters
    # must come <em>before</em> XSLT and XML filenames<mdash/>hence
    # the ordering in this command. Per <url
    # href="https://stackoverflow.com/a/9057392">SO</url>, the
    # expression <c>${@:2}</c> pass all arguments except for the
    # first one.
    xsltproc --xinclude ${@:2} ${PTX}/user/nsf-proposal-html.xsl ${PRO}/src/$1
}

#   </subsection>
#
#   <subsection>
#       <title>Build process</title>
#
# <p>Perform the build in the <c>OUT</c> directory.</p>
cd ${OUT}

banner "project proposal"
run_xsltproc proposal.xml

banner "project summary"
# The project summary must not have numbered heading, hence the
# <c>numbering.maximum.level</c>.
run_xsltproc summary.ptx --stringparam numbering.maximum.level 0

banner "data management plan"
run_xsltproc data-management-plan.ptx

# Return to the previous directory; see <url
# href="https://unix.stackexchange.com/a/330885">SO</url>.
cd -

#   </subsection>
# </section>